@@ -5,7 +5,7 @@ use warnings;
 package MongoDBx::Queue;
 
 # ABSTRACT: A message queue implemented with MongoDB
-our $VERSION = '1.001'; # VERSION
+our $VERSION = '1.002'; # VERSION
 
 use Moose 2;
 use MooseX::Types::Moose qw/:all/;
@@ -20,12 +20,19 @@ my $ID       = '_id';
 my $RESERVED = '_r';
 my $PRIORITY = '_p';
 
-with 'MooseX::Role::Logger', 'MooseX::Role::MongoDB' => { -version => 0.003 };
+with 'MooseX::Role::Logger', 'MooseX::Role::MongoDB' => { -version => 0.006 };
 
 #--------------------------------------------------------------------------#
 # Public attributes
 #--------------------------------------------------------------------------#
 
+# =attr database_name
+#
+# A MongoDB database name.  Unless a C<db_name> is provided in the
+# C<client_options> attribute, this database will be the default for
+# authentication.  Defaults to 'test'
+#
+# =cut
 
 has database_name => (
     is      => 'ro',
@@ -33,6 +40,12 @@ has database_name => (
     default => 'test',
 );
 
+# =attr client_options
+#
+# A hash reference of L<MongoDB::MongoClient> options that will be passed to its
+# C<connect> method.
+#
+# =cut
 
 has client_options => (
     is      => 'ro',
@@ -40,6 +53,12 @@ has client_options => (
     default => sub { {} },
 );
 
+# =attr collection_name
+#
+# A collection name for the queue.  Defaults to 'queue'.  The collection must
+# only be used by MongoDBx::Queue or unpredictable awful things will happen.
+#
+# =cut
 
 has collection_name => (
     is      => 'ro',
@@ -47,6 +66,12 @@ has collection_name => (
     default => 'queue',
 );
 
+# =attr safe
+#
+# Boolean that controls whether 'safe' inserts/updates are done.
+# Defaults to true.
+#
+# =cut
 
 has safe => (
     is      => 'ro',
@@ -60,7 +85,7 @@ sub _build__mongo_client_options   { $_[0]->client_options }
 sub BUILD {
     my ($self) = @_;
     # ensure index on PRIORITY in the same order we use for reserving
-    $self->mongo_collection( $self->collection_name )
+    $self->_mongo_collection( $self->collection_name )
       ->ensure_index( [ $PRIORITY => 1 ] );
 }
 
@@ -68,21 +93,77 @@ sub BUILD {
 # Public methods
 #--------------------------------------------------------------------------#
 
+# =method new
+#
+#    $queue = MongoDBx::Queue->new(
+#         database_name   => "my_app",
+#         client_options  => {
+#             host => "mongodb://example.net:27017",
+#             username => "willywonka",
+#             password => "ilovechocolate",
+#         },
+#    );
+#
+# Creates and returns a new queue object.
+#
+# =method add_task
+#
+#   $queue->add_task( \%message, \%options );
+#
+# Adds a task to the queue.  The C<\%message> hash reference will be shallow
+# copied into the task and not include objects except as described by
+# L<MongoDB::DataTypes>.  Top-level keys must not start with underscores, which are
+# reserved for MongoDBx::Queue.
+#
+# The C<\%options> hash reference is optional and may contain the following key:
+#
+# =for :list
+# * C<priority>: sets the priority for the task. Defaults to C<time()>.
+#
+# Note that setting a "future" priority may cause a task to be invisible
+# to C<reserve_task>.  See that method for more details.
+#
+# =cut
 
 sub add_task {
     my ( $self, $data, $opts ) = @_;
 
-    $self->mongo_collection( $self->collection_name )
+    $self->_mongo_collection( $self->collection_name )
       ->insert( { %$data, $PRIORITY => $opts->{priority} // time(), },
         { safe => $self->safe, } );
 }
 
+# =method reserve_task
+#
+#   $task = $queue->reserve_task;
+#   $task = $queue->reserve_task( \%options );
+#
+# Atomically marks and returns a task.  The task is marked in the queue as
+# "reserved" (in-progress) so it can not be reserved again unless is is
+# rescheduled or timed-out.  The task returned is a hash reference containing the
+# data added in C<add_task>, including private keys for use by MongoDBx::Queue
+# methods.
+#
+# Tasks are returned in priority order from lowest to highest.  If multiple tasks
+# have identical, lowest priorities, their ordering is undefined.  If no tasks
+# are available or visible, it will return C<undef>.
+#
+# The C<\%options> hash reference is optional and may contain the following key:
+#
+# =for :list
+# * C<max_priority>: sets the maximum priority for the task. Defaults to C<time()>.
+#
+# The C<max_priority> option controls whether "future" tasks are visible.  If
+# the lowest task priority is greater than the C<max_priority>, this method
+# returns C<undef>.
+#
+# =cut
 
 sub reserve_task {
     my ( $self, $opts ) = @_;
 
     my $now    = time();
-    my $result = $self->mongo_database->run_command(
+    my $result = $self->_mongo_database->run_command(
         [
             findAndModify => $self->collection_name,
             query         => {
@@ -103,10 +184,26 @@ sub reserve_task {
     }
 }
 
+# =method reschedule_task
+#
+#   $queue->reschedule_task( $task );
+#   $queue->reschedule_task( $task, \%options );
+#
+# Releases the reservation on a task so it can be reserved again.
+#
+# The C<\%options> hash reference is optional and may contain the following key:
+#
+# =for :list
+# * C<priority>: sets the priority for the task. Defaults to the task's original priority.
+#
+# Note that setting a "future" priority may cause a task to be invisible
+# to C<reserve_task>.  See that method for more details.
+#
+# =cut
 
 sub reschedule_task {
     my ( $self, $task, $opts ) = @_;
-    $self->mongo_collection( $self->collection_name )->update(
+    $self->_mongo_collection( $self->collection_name )->update(
         { $ID => $task->{$ID} },
         {
             '$unset' => { $RESERVED => 0 },
@@ -116,24 +213,53 @@ sub reschedule_task {
     );
 }
 
+# =method remove_task
+#
+#   $queue->remove_task( $task );
+#
+# Removes a task from the queue (i.e. indicating the task has been processed).
+#
+# =cut
 
 sub remove_task {
     my ( $self, $task ) = @_;
-    $self->mongo_collection( $self->collection_name )->remove( { $ID => $task->{$ID} } );
+    $self->_mongo_collection( $self->collection_name )
+      ->remove( { $ID => $task->{$ID} } );
 }
 
+# =method apply_timeout
+#
+#   $queue->apply_timeout( $seconds );
+#
+# Removes reservations that occurred more than C<$seconds> ago.  If no
+# argument is given, the timeout defaults to 120 seconds.  The timeout
+# should be set longer than the expected task processing time, so that
+# only dead/hung tasks are returned to the active queue.
+#
+# =cut
 
 sub apply_timeout {
     my ( $self, $timeout ) = @_;
     $timeout //= 120;
     my $cutoff = time() - $timeout;
-    $self->mongo_collection( $self->collection_name )->update(
+    $self->_mongo_collection( $self->collection_name )->update(
         { $RESERVED => { '$lt'     => $cutoff } },
         { '$unset'  => { $RESERVED => 0 } },
         { safe => $self->safe, multiple => 1 }
     );
 }
 
+# =method search
+#
+#   my @results = $queue->search( \%query, \%options );
+#
+# Returns a list of tasks in the queue based on search criteria.  The
+# query should be expressed in the usual MongoDB fashion.  In addition
+# to MongoDB options C<limit>, C<skip> and C<sort>, this method supports
+# a C<reserved> option.  If present, results will be limited to reserved
+# tasks if true or unreserved tasks if false.
+#
+# =cut
 
 sub search {
     my ( $self, $query, $opts ) = @_;
@@ -145,7 +271,7 @@ sub search {
         delete $opts->{reserved};
     }
     my $cursor =
-      $self->mongo_collection( $self->collection_name )->query( $query, $opts );
+      $self->_mongo_collection( $self->collection_name )->query( $query, $opts );
     if ( $opts->{fields} && ref $opts->{fields} ) {
         my $spec =
           ref $opts->{fields} eq 'HASH'
@@ -156,6 +282,18 @@ sub search {
     return $cursor->all;
 }
 
+# =method peek
+#
+#   $task = $queue->peek( $task );
+#
+# Retrieves a full copy of the task from the queue.  This is useful to retrieve all
+# fields from a partial-field result from C<search>.  It is equivalent to:
+#
+#   $self->search( { _id => $task->{_id} } );
+#
+# Returns undef if the task is not found.
+#
+# =cut
 
 sub peek {
     my ( $self, $task ) = @_;
@@ -163,16 +301,30 @@ sub peek {
     return wantarray ? @result : $result[0];
 }
 
+# =method size
+#
+#   $queue->size;
+#
+# Returns the number of tasks in the queue, including in-progress ones.
+#
+# =cut
 
 sub size {
     my ($self) = @_;
-    return $self->mongo_collection( $self->collection_name )->count;
+    return $self->_mongo_collection( $self->collection_name )->count;
 }
 
+# =method waiting
+#
+#   $queue->waiting;
+#
+# Returns the number of tasks in the queue that have not been reserved.
+#
+# =cut
 
 sub waiting {
     my ($self) = @_;
-    return $self->mongo_collection( $self->collection_name )
+    return $self->_mongo_collection( $self->collection_name )
       ->count( { $RESERVED => { '$exists' => boolean::false } } );
 }
 
@@ -187,7 +339,7 @@ __END__
 
 =pod
 
-=encoding utf-8
+=encoding UTF-8
 
 =head1 NAME
 
@@ -195,7 +347,7 @@ MongoDBx::Queue - A message queue implemented with MongoDB
 
 =head1 VERSION
 
-version 1.001
+version 1.002
 
 =head1 SYNOPSIS
 
